(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('echarts')) :
  typeof define === 'function' && define.amd ? define('bk.echarts', ['exports', 'echarts'], factory) :
  (factory((global.bk = global.bk || {}, global.bk.echarts = global.bk.echarts || {}),global.echarts));
}(this, (function (exports,echarts) { 'use strict';

echarts = 'default' in echarts ? echarts['default'] : echarts;

/**
 * @description - util method for develop
 */
/**
 * @description - check if variable is function
 *
 * @param {any} anything
 *
 * @return {boolean}
 */

function isFunction(anything) {
  return Object.prototype.toString.apply(anything) === '[object Function]';
}

/**
 * @description - generate uniq identity for easier watch
 *
 * @return {string}
 */
function uuid() {
  return Math.random().toString(36).substr(2, 9);
}

/**
 * @description - proxy echarts instance, make none-element operation available
 * @author - bornkiller <hjj491229492@hotmail.com>
 */
var _createClass$1 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck$1(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var MonkeyPatchBridge = function () {
  function MonkeyPatchBridge() {
    _classCallCheck$1(this, MonkeyPatchBridge);

    // middleware echarts instance
    this.instance = {};
    // flag stand for element initialize
    this.connected = false;
    // stock history options for controller cache or ng HMR
    this.history = {};
    // buffer event listener, stock history event listen for re-connect
    this.bufferEventListen = [];
    this.restoreBufferVariable();
    this.lazyMethodList = ['getWidth', 'getHeight', 'getDom', 'getOption', 'getDataURL', 'getConnectedDataURL', 'convertToPixel', 'convertFromPixel', 'containPixel', 'isDisposed', 'dispatchAction'];
  }

  /**
   * @description - 统一管控缓存数据
   */


  _createClass$1(MonkeyPatchBridge, [{
    key: 'restoreBufferVariable',
    value: function restoreBufferVariable() {
      // buffer instance group setting
      this.bufferGroupCategory = '';
      // buffer DOM width and height setting
      this.bufferVisionSize = [];
      // buffer loading status switcher
      this.bufferLoadingSwitchery = [];
      // buffer options setting
      this.bufferOptions = [];
    }

    /**
     * @description - 传递echarts实例
     *
     * @see - http://echarts.baidu.com/api.html#echartsInstance
     *
     * @param {object} instance
     */

  }, {
    key: 'transferCoreBridge',
    value: function transferCoreBridge(instance) {
      var _this = this;

      this.instance = instance;
      this.connected = true;

      // 延后加载非关键部分方法，属性
      this.lazyMethodList.forEach(function (key) {
        Reflect.set(_this, key, Reflect.get(_this.instance, key).bind(_this.instance));
      });
    }

    /**
     * @description - 消费实例化前缓存数据
     */

  }, {
    key: 'dealWithInventory',
    value: function dealWithInventory() {
      var _this2 = this;

      this.group = this.bufferGroupCategory;
      this.bufferVisionSize.length && this.resize.apply(this, _toConsumableArray(this.bufferVisionSize));
      this.bufferLoadingSwitchery.length && this.showLoading.apply(this, _toConsumableArray(this.bufferLoadingSwitchery));
      this.bufferOptions.length && this.bufferOptions.forEach(function (args) {
        return _this2.setOption.apply(_this2, _toConsumableArray(args));
      });
      this.bufferEventListen.length && this.bufferEventListen.forEach(function (stock) {
        return _this2.on(stock.eventName, stock.handler);
      });

      this.restoreBufferVariable();
    }

    /**
     * @description - 缓冲分组标记
     *
     * @see - http://echarts.baidu.com/api.html#echartsInstance.group
     */

  }, {
    key: 'setOption',


    /**
     * @description - 缓冲setOption调用
     *
     * @see - http://echarts.baidu.com/api.html#echartsInstance.setOption
     */
    value: function setOption() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      this.connected ? Reflect.apply(this.instance.setOption, this.instance, args) : this.bufferOptions.push(args);
      this.connected && (this.history = Reflect.apply(this.instance.getOption, this.instance, []));
      return this;
    }

    /**
     * @description - 式缓冲resize调用
     *
     * @see - http://echarts.baidu.com/api.html#echartsInstance.on
     */

  }, {
    key: 'resize',
    value: function resize() {
      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      this.connected ? Reflect.apply(this.instance.resize, this.instance, args) : this.bufferVisionSize = args;
      return this;
    }

    /**
     * @description - 缓冲事件绑定
     *
     * @see - http://echarts.baidu.com/api.html#echartsInstance.on
     *
     * @param {string} eventName
     * @param {function} handler
     *
     * @return {MonkeyPatchBridge}
     */

  }, {
    key: 'on',
    value: function on(eventName, handler) {
      if (this.connected) {
        Reflect.apply(this.instance.on, this.instance, [eventName, handler]);
      } else {
        this.bufferEventListen = [].concat(_toConsumableArray(this.bufferEventListen), [{ eventName: eventName, handler: handler }]);
      }

      return this;
    }

    /**
     * @description - 缓冲事件解绑
     *
     * @see - http://echarts.baidu.com/api.html#echartsInstance.off
     *
     * @param {string} eventName
     * @param {function} handler
     *
     * @return {MonkeyPatchBridge}
     */

  }, {
    key: 'off',
    value: function off(eventName, handler) {
      if (this.connected) {
        Reflect.apply(this.instance.off, this.instance, [eventName, handler]);
      } else {
        this.bufferEventListen = this.bufferEventListen.filter(function (stock) {
          return isFunction(handler) ? !(stock.eventName === eventName && stock.handler === handler) : !(stock.eventName === eventName);
        });
      }

      return this;
    }

    /**
     * @description - 处理controller实例重连接引发的潜在问题
     */

  }, {
    key: 'replay',
    value: function replay() {
      this.instance.setOption(this.history);
      return this;
    }

    /**
     * @description - 缓冲showLoading调用
     *
     * @see - http://echarts.baidu.com/api.html#echartsInstance.showLoading
     */

  }, {
    key: 'showLoading',
    value: function showLoading() {
      for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      this.connected ? Reflect.apply(this.instance.showLoading, this.instance, args) : this.bufferLoadingSwitchery = args.length ? args : ['default'];
      return this;
    }

    /**
     * @description - 缓冲hideLoading调用
     *
     * @see - http://echarts.baidu.com/api.html#echartsInstance.hideLoading
     */

  }, {
    key: 'hideLoading',
    value: function hideLoading() {
      this.connected ? Reflect.apply(this.instance.hideLoading, this.instance, []) : this.bufferLoadingSwitchery = [];
      return this;
    }

    /**
     * @description - 缓冲clear调用
     *
     * @see - http://echarts.baidu.com/api.html#echartsInstance.clear
     */

  }, {
    key: 'clear',
    value: function clear() {
      this.connected && this.instance.clear();
      return this;
    }

    /**
     * @description - 缓冲dispose调用
     *
     * @see - http://echarts.baidu.com/api.html#echartsInstance.dispose
     */

  }, {
    key: 'dispose',
    value: function dispose() {
      var _this3 = this;

      this.connected && this.instance.dispose();
      this.instance = {};
      this.connected = false;
      // reset buffer content
      this.restoreBufferVariable();
      // 剔除延迟加载方法
      this.lazyMethodList.forEach(function (key) {
        Reflect.deleteProperty(_this3, key);
      });
    }
  }, {
    key: 'group',
    get: function get() {
      return this.connected ? Reflect.get(this.instance, 'group') : this.bufferGroupCategory;
    },
    set: function set(value) {
      this.connected ? Reflect.set(this.instance, 'group', value) : this.bufferGroupCategory = value;
      return value;
    }
  }]);

  return MonkeyPatchBridge;
}();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @description - echarts lite data operator
 * @author - bornkiller <hjj491229492@hotmail.com>
 */

var Bridge = function (_MonkeyPatchBridge) {
  _inherits(Bridge, _MonkeyPatchBridge);

  /**
   * @description - echarts bridge instance
   *
   * @param {string} theme - echarts theme
   * @param {object} initOptions - echarts init options
   * @param {object} mediaOptions - echarts media options
   */
  function Bridge(theme) {
    var initOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var mediaOptions = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

    _classCallCheck(this, Bridge);

    var _this = _possibleConstructorReturn(this, (Bridge.__proto__ || Object.getPrototypeOf(Bridge)).call(this));

    _this.theme = theme;
    _this.initOptions = initOptions;
    _this.mediaOptions = mediaOptions;
    _this.uuid = uuid();
    return _this;
  }

  /**
   * @description - 代理echarts初始化函数，只需要传入关键element即可
   *
   * @param {HTMLDivElement|HTMLCanvasElement} element
   */


  _createClass(Bridge, [{
    key: 'connect',
    value: function connect(element) {
      if (!(element instanceof HTMLDivElement || element instanceof HTMLCanvasElement)) {
        throw new Error('HTMLElement argument required');
      }

      // 此处为部分hack操作
      // 限定媒体查询，简易声明对象时直接指定，而不是后续控制
      this.middleware = echarts.init(element, this.theme, this.initOptions);
      this.middleware.setOption({ series: [] });
      this.middleware.setOption({ media: this.mediaOptions });
      // 传递实例，消费缓存数据
      _get(Bridge.prototype.__proto__ || Object.getPrototypeOf(Bridge.prototype), 'transferCoreBridge', this).call(this, this.middleware);
      _get(Bridge.prototype.__proto__ || Object.getPrototypeOf(Bridge.prototype), 'dealWithInventory', this).call(this);

      return this;
    }

    /**
     * @description - 销毁应用实例
     */

  }, {
    key: 'disconnect',
    value: function disconnect() {
      _get(Bridge.prototype.__proto__ || Object.getPrototypeOf(Bridge.prototype), 'clear', this).call(this);
      _get(Bridge.prototype.__proto__ || Object.getPrototypeOf(Bridge.prototype), 'dispose', this).call(this);
    }
  }]);

  return Bridge;
}(MonkeyPatchBridge);

/**
 * @description - just for rollup fight
 */

exports.Bridge = Bridge;

Object.defineProperty(exports, '__esModule', { value: true });

})));
