/**
 * @description - echarts-bridge declare definition
 */

type Width = number | string | null | undefined;
type Height = number | string | null | undefined;

import {ECharts, EChartOption} from 'echarts';

// echarts init options
export declare interface EchartsInitOptions {
  devicePixelRatio?: number
  renderer?: string
  width?: number|string
  height?: number | string
}

// echarts media query definition
export declare interface EchartsMediaQuery {
  width?: number,
  minWidth?: number,
  maxWidth?: number,
  height?: number,
  minHeight?: number,
  maxHeight?: number,
  aspectRatio?: number
  minAspectRatio?: number
  maxAspectRatio?: number
}

// echarts media match and rule
export declare interface EchartsMediaRule {
  query?: EchartsMediaQuery;
  option: EChartOption
}

// echarts resize options
export declare interface EchartsResizeOptions {
  width?: Width;
  height?: Height;
  silent?: boolean;
}

// pixel坐标转换关联方法使用
export declare interface EchartsPixelFinder {
  seriesIndex?: number,
  seriesId?: string,
  seriesName?: string,
  geoIndex?: number,
  geoId?: string,
  geoName?: string,
  xAxisIndex?: number,
  xAxisId?: string,
  xAxisName?: string,
  yAxisIndex?: number,
  yAxisId?: string,
  yAxisName?: string,
  gridIndex?: number,
  gridId?: string
  gridName?: string
}

// proxy echarts instance
declare class MonkeyPatchBridge {
  constructor();

  private instance: Object;
  private connected: boolean;
  private history: EChartOption;
  private bufferGroupCategory: string;
  private bufferVisionSize: [EchartsResizeOptions];
  private bufferEventListen: Array<{eventName: string, handler: Function}>;
  private lazyMethodList: Array<string>;

  private restoreBufferVariable(): void;

  protected transferCoreBridge(instance: ECharts): void;

  protected dealWithInventory(): void;

  // echarts instance below
  group: string;

  getWidth(): number;

  getHeight(): number;

  getDom(): HTMLCanvasElement|HTMLDivElement;

  getOption(): Object;

  getDataURL(opts: {
    // 导出的格式，可选 png, jpeg
    type?: string,
    // 导出的图片分辨率比例，默认为 1。
    pixelRatio?: number,
    // 导出的图片背景色，默认使用 option 里的 backgroundColor
    backgroundColor?: string,
    // 忽略组件列表
    excludeComponents: Array<string>
  }): string;

  getConnectedDataURL(opts: {
    // 导出的格式，可选 png, jpeg
    type?: string,
    // 导出的图片分辨率比例，默认为 1。
    pixelRatio?: number,
    // 导出的图片背景色，默认使用 option 里的 backgroundColor
    backgroundColor?: string,
    // 忽略组件列表
    excludeComponents: Array<string>
  }): string;

  isDisposed(): boolean;

  dispatchAction(payload: Object): void;

  convertToPixel(finder: EchartsPixelFinder, value: Array<number>|string): Array<number>|string;

  convertFromPixel(finder: EchartsPixelFinder, value: Array<number>|string): Array<number>|string;

  containPixel(finder: EchartsPixelFinder, value: Array<number>): boolean;

  setOption(option: EChartOption, notMerge?: boolean, lazyUpdate?: boolean): MonkeyPatchBridge;

  resize(options: EchartsResizeOptions): MonkeyPatchBridge

  on(eventName: string, handler: Function): MonkeyPatchBridge;

  off(eventName: string, handler: Function): MonkeyPatchBridge;

  replay(): void;

  showLoading(): MonkeyPatchBridge;

  hideLoading(): MonkeyPatchBridge;

  clear(): MonkeyPatchBridge;

  dispose(): void;
}

export declare class Bridge extends MonkeyPatchBridge {
  constructor(theme: string, initOptions: EchartsInitOptions, mediaOptions: Array<EchartsMediaRule>);

  connect(element: HTMLDivElement|HTMLCanvasElement): Bridge;

  disconnect(): void;
}